/*
  Copyright (C) 2001 Jesper Hansen <jesperh@telia.com>.

  Rewritten by: Nikolai Vorontsov <nickviz@mail.be>

  Rewritten by:	Romuald Bialy (aka MIS) <romek_b@o2.pl>

  This file is part of the yampp system.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software Foundation, 
  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
	PIN assignements
	
	PA0-PA7		data bus 0..7 + Address 0..7
	PC0-PC7		address 8..15	
	
	PB0		T0	DIOW		ATA out
	PB1		T1	DIOR		ATA out
	PB2		AIN0	DREQ		VS1001 in		
	PB3		AIN1	BSYNC		VS1001 out
	PB4		SS	MP3		VS1001 out
	PB5		MOSI	SO		VS1001 out
	PB6		MISO 	SI		VS1001 in
	PB7		SCK	SCK		VS1001 out

	PD0		RxD	RS-232		UART in
	PD1		TxD	RS-232		UART out
	PD2		INT0	IR_INPUT	IR in
	PD3		INT1	KEY_INPUT	KEY in
	PD4		T0		
	PD5		T1	LCD_ENABLE	LCD out
	PD6		WR	WR		RAM out
	PD7		RD	RD		RAM out
*/


#include <stdlib.h>
#include <io.h>
#include <progmem.h>
#include <eeprom.h>
#include <sig-avr.h>
#include <interrupt.h>
#include <wdt.h>
#include <string.h>

#include "Constants.h"
#include "types.h"
#include "mem.h"
#include "delay.h"
#include "ata_if.h"
#include "fat.h"
#include "vs1001.h"
#include "remote.h"
#include "uart.h"
#include "lcd.h"


// Amount of small steps in LCD play position bar
#ifdef PROGRESBAR_II
 #if (LCD_LINES == 2)
  #define LCD_STEPS ((LCD_LINE_LENGTH - 7 - 2*(LCD_LINE_LENGTH != 16)) * 5)
 #else
  #define LCD_STEPS ((LCD_LINE_LENGTH - 6) * 5)
 #endif
#else //PROGRESBAR_II
 #if (LCD_LINES == 2)
  #define LCD_STEPS ((LCD_LINE_LENGTH - 6) * 5)
 #else
  #define LCD_STEPS ((LCD_LINE_LENGTH - 5) * 5)
 #endif
#endif //PROGRESBAR_II

// Amount of buffers writed before relinquish control
#define MAX_CONS_WRITES 100


#define	EV_IDLE		0
#define	EV_PLAY		1
#define	EV_STOP		2
#define	EV_NEXT		3
#define	EV_PREV		4
#define	EV_NEXT10	5
#define	EV_PREV10	6
#define	EV_VOLUP	7
#define	EV_VOLDN	8
#define	EV_NEXTD	9
#define	EV_PREVD	10
#define	EV_FFWD		11
#define	EV_FREW		12
#define	EV_LOUDNESS	13
#define	EV_RANDOM	14
#define	EV_REMAIN	15
#define	EV_NEXTL	16
#define	EV_PREVL	17
#define	EV_NUMBER	18
#define	EV_PAUSE	19

// Auxilliary functions declarations
void sectordump(u32 sector);
u08* unsigned2str(u16 track, u08 *str);
u08 get_char_event(void);
u08 ir_event(void);
u08 get_key_event(void);
void setvolume(u08 v);
void dispbar(void);
u16 Numb2Int(void);
void BadHeadStrip(void);
u08 get_char_event(void);
void send_sinewave_beeps(void);

#ifdef PROGRESBAR_II
 void lcd_frame(void);
#else
 #define lcd_frame()
#endif


// Global variables
u08 scroll_length;
u08 scroll_pos;
char *scroll_line;
u08 LastEvent		= EV_IDLE;

#if (LCD_LINES == 4) && (defined DIR_SCROLL)
 u08 dir_scroll_length;
 u08 dir_scroll_pos;
 char *dir_scroll_line;
#endif

bool bPlaying;				// Current state - play/stop
u08 *pPlayData;				// Pointer to current position in currently used buffer
u08 *pUpdBuf;				// Pointer to buffer need updating

u32 dwPlayed;				// Amount of already played blocks (32 bytes)
u16 wBackStepPos;			// Actual position in backstep tabele
u16 wSongNum;				// Current playing song
u16 wMaxSongs = 1;			// Maximum available songs

#define TI1_H	(((u16)-(F_CPU / 2560)) >> 8)
#define TI1_L	(((u16)-(F_CPU / 2560)) & 0xff )

volatile u08 nDispUpd = 0;			// Counter for display update loop
volatile u08 nKeyTime = 0;			// Counter for keypress time check - Added by MIS

#ifdef OLD_VS1001
 volatile u16 wPlayT = 0;			// Play Time counter for older VS1001 chips
#endif

SIGNAL(SIG_OVERFLOW1)				// Timer1 overflow every 100 mS
{
	nDispUpd++;
	nKeyTime++;
#ifdef OLD_VS1001
	wPlayT++;
#endif
	outp(TI1_H, TCNT1H);			// Reload timer
	outp(TI1_L, TCNT1L);
}

#ifndef SETUP_REMOTE_CODES

// Random stuff, maximum 65535 songs on the disk!

static u32 seed = 1;
static u16 wFirstInDir = 0;

// Init randomize table
void InitRnd(void)
{
u16 Temp;
	for (Temp = 0; Temp < (wMaxSongs >> 3); Temp++)
		RANDOM_TAB[Temp] = 0;					// one bit in table represent one song
	RANDOM_TAB[Temp] = (0xff << (wMaxSongs % 8));			// last byte.
}

u16 randcalc(u16 max)							// assembler optimized randomizer function
{
//  return ((seed = seed * 1103515245L + 12345) % max);	// get new song number

	asm volatile ("push r24");
	asm volatile ("push r25");
	asm volatile ("lds r22,seed");
	asm volatile ("lds r23,(seed)+1");
	asm volatile ("lds r24,(seed)+2");
	asm volatile ("lds r25,(seed)+3");
	asm volatile ("ldi r18,lo8(1103515245)");
	asm volatile ("ldi r19,hi8(1103515245)");
	asm volatile ("ldi r20,hlo8(1103515245)");
	asm volatile ("ldi r21,hhi8(1103515245)");
#ifdef YAMPP3USB
	asm volatile ("call __mulsi3");
#else
	asm volatile ("rcall __mulsi3");
#endif
	asm volatile ("subi r22,lo8(-(12345))");
	asm volatile ("sbci r23,hi8(-(12345))");
	asm volatile ("sbci r24,hlo8(-(12345))");
	asm volatile ("sbci r25,hhi8(-(12345))");
	asm volatile ("sts seed,r22");
	asm volatile ("sts (seed)+1,r23");
	asm volatile ("sts (seed)+2,r24");
	asm volatile ("sts (seed)+3,r25");
	asm volatile ("pop r19");
	asm volatile ("pop r18");
	asm volatile ("clr r20");
	asm volatile ("clr r21");
#ifdef YAMPP3USB
	asm volatile ("call __udivmodsi4");
#else
	asm volatile ("rcall __udivmodsi4");
#endif
	asm volatile ("mov r25,r23");
	asm volatile ("mov r24,r22");
	return max;
}

u08 tabval(u16 num, u08 fn)					// check random tabele for song status
{									// and mark as played if necessary
	u16 index = (num + wFirstInDir) >> 3;
	u08 offset = 1 << ((num  + wFirstInDir) % 8);
	if(fn)
		RANDOM_TAB[index] |= offset;			// if fn=1 mark as played
	return (RANDOM_TAB[index] & offset);		// return status
}

// new random procedure
u16 do_rand(u16 max)						// returned random value from 0 to max - 1
{
u16 num;
	for(num = 0; num < max; num++)
		if(tabval(num,0) == 0)
			break;
	if (num == max)
	{
		InitRnd();
		num = -1;
	}
	else
	{
		do
		{
			WDR;
			num = randcalc(max);
		} while (tabval(num,0) != 0);			// check this song not be played piervously
		tabval(num,1);					// mark as just played
	}
	return num;
}

// This function return number of songs in current direcrory and sets first song number variable
u16 checkdir(void)
{
	register u16 Temp;
	for (Temp = 0; Temp < nLastPosD; Temp++)				// search dir tabele
	{
		if (DirsTable[Temp].song_num <= wSongNum)
			wFirstInDir = DirsTable[Temp].song_num;			// update first song number in current dir
		else
			break;							// found end of current dir
	}
	if (Temp == nLastPosD)							// if this is last dir on CD
		return (wMaxSongs - wFirstInDir);
	return (DirsTable[Temp].song_num - wFirstInDir);
}

#if (LCD_LINES == 4)
void printsp(void)
{
	register u08 i = (LCD_LINE_LENGTH - 11);
	while(i--) lcd_putchar(' ');
}
#endif

void load_buffers(void)
{
	ReadBuffer(BUFFER1P);			// Load buffer1
	ReadBuffer(BUFFER2P);			// Load buffer2
	pUpdBuf = 0;				// Mark as just loaded
	pPlayData = BUFFER1P;			// Start playing from buffer1
#ifdef OLD_VS1001
	BadHeadStrip();
#endif				
}



u08 *randstr  = "Rand*";
u08 *loudstr  = "Loud*";
u08 randmod[] = "-+*";
u08 trackstr[6], *pTrack, timestr[6], tot_sw = 0;

#ifdef ENABLE_NUMERIC
u08 number[6], *pNum = number;
#endif

#endif // #ifndef SETUP_REMOTE_CODES
//----------------------------------------------------------------------------
// Main part
//----------------------------------------------------------------------------
int main(void)
{
#ifndef SETUP_REMOTE_CODES
	bool bRandom, bLoudness;				// Modificators
	bool bFfwdMute = false;					// Search mute flag
	u08 nVolume = 0;					// Current volume level
	u16 wPlayTime;						// Current play time
#ifdef REMAIN_TIME
	u16 wSongTime = 0;					// Song time for remain calculation
	u08 bTimeMode = 0;					// Remain time modificator
#endif
	u16 wAddTime = 0;					// Fix play time for search
	u08 nWrites, i;						// Buffering variables
	u08 event = EV_STOP;					// Current event
	u08 *src, *dst; 					// Temporary variables
	u16 nTemp;
	udiv_t divt;
#endif // #ifndef SETUP_REMOTE_CODES
//----------------------------------------------------------------------------
// B - Port
//----------------------------------------------------------------------------
	outp(0x57, PORTB);
	outp(0xBB, DDRB);

//----------------------------------------------------------------------------
// D - Port
//----------------------------------------------------------------------------
	outp (0xDC ,PORTD);
#ifdef ENABLE_RST_OUT
	outp (0xF0 ,DDRD);
#else
	outp (0xE0 ,DDRD);
#endif

#ifdef YAMPP3USB
#define SRW	SRW10
	cbi(DDRD, PD5);		// PD5 is input in yampp-3/USB

//----------------------------------------------------------------------------
// E - Port
//----------------------------------------------------------------------------
	cbi(DDRE, PE0);		// PE0 is IR input
	sbi(PORTE, PE0);	// activate pullup
	sbi(DDRE, PE1);		// set ALE pin as output
	cbi(PORTE, PE1);	// and set it low (inactive)
	sbi(DDRE, PE2);		// PE2 is LCD_ENABLE
	cbi(PORTE, PE2);	// and set it low (inactive)
#endif

//----------------------------------------------------------------------------
// Timer 1
//----------------------------------------------------------------------------
	// setup timer1 for a 100mS periodic interrupt
	outp(0, TCCR1A);
	outp(4, TCCR1B);			// Prescaler /256  tPeriod = 32 uS
	outp(TI1_H, TCNT1H);			// Load counter value hi
	outp(TI1_L, TCNT1L);			// Load counter value lo

//----------------------------------------------------------------------------
// Initialization
//----------------------------------------------------------------------------

	outp((1<<SRE)|(1<<SRW), MCUCR);		// enable external SRAM and wait states 

	delay10(1000);
   	lcd_init(0, LCD_FUNCTION_DEFAULT);

	outp((1<<MSTR) | (1<<SPE), SPCR);	// SPI 2 MHz clock
	inp(SPSR);				// clear SPI status
#if (LCD_LINE_LENGTH == 16)
	lcd_puts_p(PSTR(" Hi, MP3 world!\n yampp-3 player"));
#endif
#if (LCD_LINE_LENGTH == 20)
	lcd_puts_p(PSTR("   Hi, MP3 world!\nyampp-3 player Alive"));
#endif
#if (LCD_LINE_LENGTH == 24)
	lcd_puts_p(PSTR("     Hi, MP3 world!\n yampp-3 player Alive !"));
#endif

	UART_Init();				// Init RS-232 link

#ifdef ENABLE_RST_OUT
	cbi(PORTD,PD4);				// activate reset signal to VS1001 and CD-ROM
	delay10(1000);
	sbi(PORTD,PD4);				// deactivate reset signal to VS1001 and CD-ROM
#endif
	// Say hello to UART
 	PRINT_p(PSTR("\n\nyampp-3 "__DATE__" "__TIME__"\nATA checking..."));

#ifdef ENABLE_WATCHDOG
	wdt_enable(0x07);
#endif

	delay10(10000);
	vs1001_setcomp(F_VS1001);		// set frequency of vs1001 clock
	vs1001_reset(false);			// Init VS1001

#ifdef SETUP_REMOTE_CODES
	setvolume(6);
	send_sinewave_beeps();			// Send three sinewave beeps to indicate
	remote_setup();				// Enter remote setup procedure
	while(1) WDR;

#else // SETUP_REMOTE_CODES	-	for get full code space

	ATA_Init();				// Init and wait for drive
	ATA_SW_Reset();				// Reset drive
	PRINT_p(PSTR("done.\n"));

	if ((nTemp = init_fat()))				// Read disk structure
	{
		lcd_clrscr();
		lcd_puts_p(PSTR("Wrong cluster\nsize "));	// Error info, recompile code
		lcd_puts(unsigned2str((nTemp>>1), trackstr));	// Show cluster size for this disk
		lcd_putchar('k');
		while (true) WDR;				// Lock player
	}

	PRINT_p(PSTR("Scanning..."));
//	wMaxSongs = dirlist(DIRLIST_VERBOSE);
	wMaxSongs = dirlist(DIRLIST_SCAN);

	PRINT(unsigned2str(wMaxSongs , trackstr));		// jman number of songs in decimal
	PRINT_p(PSTR(" entries\n"));

	seed = 1 + inp(TCNT1L);					// Init randomizer from timer 
								// (ATA init gives us random delay)
	InitRnd();						// Init randomizer tabele

	if ((nVolume = eeprom_rb(EEPROM_VOLUME)) > MIN_VOLUME)	// Load and set initial volume
		nVolume = MIN_VOLUME / 2;			// set volume to 1/2 scale after eeprom erase

	setvolume(nVolume);

#ifdef PWR_BEEPS
	send_sinewave_beeps();					// Send three sinewave beeps to indicate
#endif								// we're starting up okay

	// load bAutoPlay and position settings
	bRandom		= eeprom_rb(EEPROM_RANDOM) % 3;		//  Load random setting
	wSongNum	= eeprom_rw(EEPROM_SONGPOS);		//  Load position setting
	bLoudness	= eeprom_rb(EEPROM_LOUDNESS);	

	// set initial status in the strings
	*(loudstr+4) = bLoudness ? '+' : '-';
	*(randstr+4) = randmod[bRandom];
	if (eeprom_rb(EEPROM_AUTOPLAY))				// Load autoplay status
		event = EV_PLAY;

//----------------------------------------------------------------------------
// start things rolling
//----------------------------------------------------------------------------
	sbi(TIMSK, TOIE1);	 			// Enable timer1 interrupt
	while (true)					// Main loop
	{
		WDR;					// Give watchdog full time
		if (UART_HasChar())			// Check all input controls
			event = get_char_event();

		if (rec80_active())
			event = ir_event();
		else
			if (nKeyTime > 16)
				LastEvent = EV_IDLE;

		if (event == EV_IDLE)
			event = get_key_event();
			
		if (bPlaying && event == EV_PLAY)	// second play press does pause
			event = EV_PAUSE;

		switch (event)				// According input control
		{
			case EV_IDLE:
				break;

			case EV_PREVD:			// Switch to previous start dir
				checkdir();
				wSongNum = ((wFirstInDir) ? wFirstInDir : wMaxSongs) -1;
				checkdir();
				wSongNum = wFirstInDir;
				goto PLAY;

			case EV_NEXTD:						// Switch to next dir
				wSongNum = checkdir() + wFirstInDir; 
				goto PLAY;

#ifdef ENABLE_NAVL
			case EV_PREVL:			// Switch to previous start letter 

				nTemp = nLastPosL - 1;
				while (nTemp < nLastPosL)
				{
					if (LetterTable[nTemp].letter < *LONG_NAME)
						break;
					else
						nTemp--;
				}
				if (nTemp >= nLastPosL)
					nTemp = nLastPosL - 1;

				wSongNum = LetterTable[nTemp].song_num;
				goto PLAY;

			case EV_NEXTL:						// Switch to next start letter
				for (nTemp = 1; nTemp < nLastPosL; nTemp++)
				{
					if (LetterTable[nTemp].letter > *LONG_NAME)
						break;
				}
				if (nTemp >= nLastPosL)
					nTemp = 0;
				wSongNum = LetterTable[nTemp].song_num;
				goto PLAY;
#endif

#ifdef ENABLE_NUMERIC
			case EV_NUMBER:
				pTrack = number;
				tot_sw = 10;
				goto DISPUPD;
#endif

			case EV_PREV10:	wSongNum -= 10;	goto PLAY;

			case EV_NEXT10:	wSongNum += 10;	goto PLAY;

			case EV_PREV:
#ifdef NEW_PREV
 				if(dwPlayed > 0x0600)
					goto PLAY;
#endif
				if (bRandom == 0)			// in normal mode
				{
					if (wSongNum == 0)
						wSongNum = wMaxSongs - 1;
					else
						wSongNum--;
					goto PLAY;
				}
				else 
					goto RAND;			// in any random mode

			case EV_NEXT: 
NEXT_SONG:			if (bRandom == 0)
				{
					wSongNum++;			// in normal mode
					goto PLAY;
				}
RAND:				switch (bRandom)
				{

					case 1:	wSongNum = do_rand(wMaxSongs);	// random from all songs on disk
						break;

					case 2:	nTemp = do_rand(checkdir());		// random from current directory
						if (nTemp == -1)			// no more songs in current dir.
						{
							nTemp = checkdir();		// calculate number of first song
							wSongNum = nTemp + wFirstInDir;	// in next directory
							if (wSongNum >= wMaxSongs)
								wFirstInDir = wSongNum = 0;
							nTemp = do_rand(checkdir());	// random from this directory
						}
						wSongNum = nTemp + wFirstInDir;		// set new song number
						break;
				}
				// fall through

			case EV_PLAY:
				eeprom_wb(EEPROM_AUTOPLAY,true);	// Write autoplay flags
PLAY:
				lcd_clrscr();
				lcd_puts_p(PSTR("Loading..."));
				vs1001_reset(bLoudness);		// Mandatory!

				if (wSongNum >= wMaxSongs)		// Protect for song number error
					wSongNum = 0;

				if (!get_dir_entry(wSongNum))		// Find song
				{
					if (!get_dir_entry(wSongNum))	// Not found ??? Try again.
					{	wSongNum = 0;		// Something wrong
						continue;
					}
				}
				eeprom_ww(EEPROM_SONGPOS, wSongNum);
				PRINT_p(PSTR("\nPlaying #"));
				pTrack = unsigned2str(wSongNum + 1, trackstr); 
				PRINT(pTrack); 
				UART_SendByte(' ');
#if (LCD_LINES == 2)
				scroll_line = src = strcat(DIR_NAME, LONG_NAME);
#else
				scroll_line = src = LONG_NAME;
#ifdef DIR_SCROLL
				dir_scroll_line = DIR_NAME;
				dir_scroll_length = strlen(DIR_NAME);
				dir_scroll_pos = 0;
#endif
#endif
				scroll_pos = 0;
				scroll_length = 4;			// To add ' >> ' sign
				while (*src++)
					scroll_length++;
				src--;

				lcd_clrscr();
#if (LCD_LINES == 2)
				PRINT(scroll_line);			// directory and file name
				lcd_puts(scroll_line);
#endif
				if(scroll_length > LCD_LINE_LENGTH+4)	// only if scrolling needed
				{
					*((u32*)src)++ = 0x203E3E20L;	// adding " >> " sign
					*src = 0;
				}
#if (LCD_LINES == 4)
				// dir name centering
#ifdef DIR_SCROLL
				if(dir_scroll_length < LCD_LINE_LENGTH)	// only if no dir name scrolling
					lcd_gotoxy(LCD_LINE_LENGTH/2 - (dir_scroll_length+1)/2, 0);
#else //DIR_SCROLL
				i= strlen(DIR_NAME);
				if(i < LCD_LINE_LENGTH)			// only if no dir name scrolling
					lcd_gotoxy(LCD_LINE_LENGTH/2 - (i+1)/2, 0);
#endif //DIR_SCROLL
				lcd_puts(DIR_NAME);			// display directory
				PRINT(DIR_NAME);
				PRINT_p(PSTR(" / "));
				lcd_gotoxy(0,1);			// Second line = filename
				lcd_puts(LONG_NAME);			// Just display it 
				PRINT(LONG_NAME);
#endif // (LCD_LINES == 4)
				wAddTime = 0;
				dwPlayed = 0;
#ifdef REMAIN_TIME
				wSongTime = (u16)(dwFileSize / (4 * 125)); // Song time calculation
#endif
				nDispUpd = tot_sw = 10;
				load_buffers();
				bPlaying = true;
#ifdef OLD_VS1001
				wPlayT = 0;
#endif				
				lcd_frame();
				break;

			case EV_STOP:
				vs1001_reset(0);			// VS1001 reset for flush buffer
				bPlaying = false;
				lcd_clrscr();
				lcd_puts_p(PSTR(" yampp-3   STOP"));
				eeprom_wb(EEPROM_AUTOPLAY,false);	// write autoplay flag
				LastEntry = -1;
				ATA_SpinDown();				// spindown HDD
				break;

			case EV_VOLDN:
				if (nVolume <= MIN_VOLUME-VOL_STEP) 
					nVolume += VOL_STEP;
				else
					nVolume = MIN_VOLUME;
				goto SETVOL;

			case EV_VOLUP:
				if (nVolume >= VOL_STEP) 
					nVolume -= VOL_STEP;
				else
					nVolume = 0;
SETVOL:				setvolume(nVolume);
				break;

			case EV_LOUDNESS:
				bLoudness = !bLoudness;
				*(loudstr+4) = bLoudness ? '+' : '-';
				vs1001_write(VS1001_MODE, 0x0080 * (bLoudness == true)); // write bit SM_BASS to VS1001K
				eeprom_wb(EEPROM_LOUDNESS, bLoudness);
				tot_sw = 14;				
				goto DISPUPD;
#ifdef REMAIN_TIME
			case EV_REMAIN:
				bTimeMode = !bTimeMode;
				tot_sw = 0;
				goto DISPUPD;
#endif				
			case EV_RANDOM:
				bRandom = (bRandom + 1) % 3 ;			// change random mode from 0 to 2
				*(randstr+4) = randmod[bRandom];
				eeprom_wb(EEPROM_RANDOM, bRandom);
				tot_sw = 12;				
DISPUPD:			nDispUpd = 10;
				break;

//jesper 011207	start			
			case EV_PAUSE:
				event = EV_IDLE;
#if (LCD_LINES == 2)
				lcd_gotoxy(LCD_LINE_LENGTH - 6 - 2*(LCD_LINE_LENGTH != 16),1);	// 2 line
				lcd_puts_p(PSTR("Paused"));
#else
				lcd_gotoxy(0,2);			// 3 line
				lcd_puts_p(PSTR("Paused"));
				printsp();
#endif
				while (rec80_active()) ;		// wait for key release
				do
				{
					WDR;				// Give watchdog full time
					if (UART_HasChar())		// Check all input controls
						event = get_char_event();
					if (rec80_active())
						event = ir_event();
					if (event == EV_IDLE)
						event = get_key_event();
				} while (event == EV_IDLE);
#if (LCD_LINES == 4)
#ifdef PROGRESBAR_II
				lcd_frame();
#else
				lcd_gotoxy(0,2);			// 3 line
				printsp();
#endif
#endif //(LCD_LINES == 4)
				break;
//jesper 011207 end	

#ifdef FFWD_SEARCH
			case EV_FFWD:
				for (i = 0; i < 0x18000 / CLUSTER_SIZE; i++)	// Number of clusters to skip 6 second
				{
					if (!ReadNextCluster())			// Skip clusters,
						goto NEXT_SONG;			// Play next song
					dwPlayed += CLUSTER_SIZE / 32;		// Update pumped bytes counter
				}
				wAddTime += 6;					// Its accurate only for 128kbps files
										// and no room for best calculation
										// on VBR files its totally imposible
SEARCHGO:			vs1001_setvolume(nVolume+40, nVolume+40);	// Decrease volume by 20dB for eliminate 
										// sound scratches
				bFfwdMute = true;
				load_buffers();
				tot_sw = 0;
				goto DISPUPD;
#endif // FFWD_SEARCH

#ifdef FREW_SEARCH
			case EV_FREW:
				if (wBackStepPos > 0)
				{
					dwCurrentCluster = Rev_Table[--wBackStepPos];
					dwPlayed = wBackStepPos * (0x18000 / 32);
					wAddTime -= 5;
					lcd_frame();
					goto SEARCHGO;
				}
#endif
			default:
				break;
		} // switch (event)

		event = EV_IDLE;			// We always go back to IDLE when
							//  exiting switch

									
		if (bPlaying)
		{
#ifdef ENABLE_NUMERIC
			if (LastEvent == EV_NUMBER && (nKeyTime > 14 || pNum == number+4))
			{
				LastEvent = EV_IDLE;
				nTemp = Numb2Int();
				pNum = number;
				*pNum = 0;
				if (nTemp <= wMaxSongs)
				{
					wSongNum = nTemp-1;
					goto PLAY;
				}
				pTrack = unsigned2str(wSongNum + 1, trackstr);
			}
#endif

//-------------------READ MPEG STREAM FROM HDD AND SENDING DATA TO VS1001 ---------------------------

			nWrites = MAX_CONS_WRITES;				// This count prevents indefinite
										// data sending to VS1001
			while (bit_is_set(PINB, PB2) && pUpdBuf == 0)
			{
				vs1001_send_32(pPlayData);			// Feed data to VS1001
				pPlayData += 32;				// Advance pointer
				if (pPlayData == BUFFER2P+BUFFER_SIZE) 		// End of Buffer 2
				{
					pPlayData = BUFFER1P;			// Pointer to Start buffer1
					pUpdBuf = BUFFER2P;			// Reload Buffer2
				}
				else if (pPlayData == BUFFER1P+BUFFER_SIZE)	// End of Buffer 1
				{
					pUpdBuf = BUFFER1P;			// Reload Buffer1
				}

				dwPlayed++;					// Count already pumped blocks (32 bytes)

				if (--nWrites == 0)				// if vs1001 needs to more data
				{
					vs1001_reset(bLoudness);		// try resetting it
					break;
				}
			}

			if (bFfwdMute && (nDispUpd == 3))			// If fast forward volume downed
			{
				vs1001_setvolume(nVolume,nVolume);		// Restore volume
				bFfwdMute = false;
			}

			i = CheckAndAdvanceCluster();
			if (pUpdBuf && i != NO_DATA)				// if any buffer need reload
			{
				ReadBuffer(pUpdBuf);				// reload buffer
				pUpdBuf = 0;					// mark as just loaded
			}
#ifdef CHECK_FILESIZE
			if ((i == NO_DATA && pUpdBuf) || (dwPlayed >= dwFileSize))	// No more data to play
#else
			if (i == NO_DATA && pUpdBuf)					// No more data to play
#endif
			{							// Try to autoplay next song
				event = EV_NEXT;
				continue;
			}


			if (nDispUpd >= 5)					// Update display info every 500ms
			{
#ifdef ENABLE_LCD
				if (scroll_length > LCD_LINE_LENGTH+4)		// Scroll song name every 500ms
				{						// if its length is more than lcd length
	 				lcd_gotoxy(0, LCD_LINES/2 - 1);
					for (nTemp = 0; nTemp < LCD_LINE_LENGTH; nTemp++)
						lcd_putchar(scroll_line[(scroll_pos + nTemp) % scroll_length]);
					scroll_pos++;
				}

#if (LCD_LINES == 4) && (defined DIR_SCROLL)

				if (dir_scroll_length > LCD_LINE_LENGTH)	// Scroll dir name 
				{						// if its length is more than LCD row chars
					lcd_gotoxy(0,0);
					for (nTemp = 0; nTemp < LCD_LINE_LENGTH; nTemp++)
						lcd_putchar(dir_scroll_line[(dir_scroll_pos + nTemp) % dir_scroll_length]);
					dir_scroll_pos++;
				}
#endif

				dispbar();
				dst = timestr;

//--------------------------------------------------------------------------------------------------------------------
#if (LCD_LINES == 2)
#ifdef REMAIN_TIME
				lcd_gotoxy(LCD_LINE_LENGTH - 7 - 2*(LCD_LINE_LENGTH != 16), 1);	// Line 2
#else
				lcd_gotoxy(LCD_LINE_LENGTH - 6 - 2*(LCD_LINE_LENGTH != 16), 1);	// Line 2
#endif
				if (tot_sw < 10)	// 0-9
				{
					src = timestr;
#ifdef OLD_VS1001
					wPlayTime = wPlayT / 10 + wAddTime;	// seconds
#ifdef FREW_SEARCH
					if (wPlayTime > 32768) wAddTime = -(wPlayT / 10);
#endif

#else  //OLD_VS1001
					wPlayTime = vs1001_read(VS1001_PLAYTIME) + wAddTime;
#endif  //OLD_VS1001

#ifdef REMAIN_TIME
					if (bTimeMode)
					{
						wPlayTime = wSongTime - wPlayTime;
						lcd_putchar('-');
					}
					else
						lcd_putchar(' ');
#endif
					if (wPlayTime < 0) wPlayTime = 0;
					divt = udiv(wPlayTime, 600);
					*src++ = '0' + (u08)divt.quot % 10;	 // song time up to 99 min
					divt = udiv(divt.rem, 60);
					*src++ = '0' + (u08)divt.quot;
					*src++ = ':';
					divt = udiv(divt.rem, 10);
					*src++ = '0' + (u08)divt.quot;
					*src++ = '0' + (u08)divt.rem;			
					*src = 0;
				}
				else if (tot_sw < 12)	// 10-11
				{
					*dst++ = '#';
					src = pTrack;
					while ((*dst = *src++)) dst++;
					while (dst < &timestr[5])
						*dst++ = ' ';
					dst = timestr;
				}
				else if (tot_sw < 14)	// 12-13
				{
					dst = randstr;
				}
				else if (tot_sw < 16)	// 14-15
				{
					dst = loudstr;
				}
	
#ifdef REMAIN_TIME
				if (tot_sw > 9)
						lcd_putchar(' ');
#endif

				tot_sw += 1;
				if (tot_sw >= 16)
					tot_sw = 0;
				lcd_puts(dst);
#if (LCD_LINE_LENGTH != 16)
				lcd_putchar(' ');		// separator
				lcd_putchar(1);			// speaker sign
#endif

#endif  //(LCD_LINES == 2)

//--------------------------------------------------------------------------------------------------------------------
#if (LCD_LINES == 4)
#ifdef REMAIN_TIME
				lcd_gotoxy(LCD_LINE_LENGTH - 6,2);			// Line 3
#else
				lcd_gotoxy(LCD_LINE_LENGTH - 5,2);			// Line 3
#endif
				src = timestr;
#ifdef OLD_VS1001
				wPlayTime = wPlayT / 10 + wAddTime;	// seconds
#ifdef FREW_SEARCH
				if (wPlayTime > 32768) wAddTime = -(wPlayT / 10);
#endif
#else  //OLD_VS1001
				wPlayTime = vs1001_read(VS1001_PLAYTIME) + wAddTime;
#endif  //OLD_VS1001

#ifdef REMAIN_TIME
				if (bTimeMode)
				{
					wPlayTime = wSongTime - wPlayTime;
					lcd_putchar('-');
				}
				else
					lcd_putchar(' ');
#endif
				if (wPlayTime < 0) wPlayTime = 0;
				divt = udiv(wPlayTime, 600);
				*src++ = '0' + (u08)divt.quot % 10;	// song time up to 99 min
				divt = udiv(divt.rem, 60);
				*src++ = '0' + (u08)divt.quot;
				*src++ = ':';
				divt = udiv(divt.rem, 10);
				*src++ = '0' + (u08)divt.quot;
				*src++ = '0' + (u08)divt.rem;			
				*src = 0;

				lcd_puts(dst);			// display time, line 3, x= 15 - 19
				lcd_putchar('\n');		// goto line 4

				dst = timestr;
				*dst++ = '#';
				src = pTrack;
				while ((*dst = *src++)) dst++;
				while (dst < &timestr[5])
					*dst++ = ' ';
				lcd_puts(timestr);		// display track number, x= 0 - 4
#if (LCD_LINE_LENGTH != 16)
				lcd_putchar(' ');		// space
#endif
				lcd_puts(randstr);		// display random status, x= 6 - 10
#if (LCD_LINE_LENGTH != 16)
				lcd_putchar(' ');		// space
#endif
				lcd_puts(loudstr);		// display loudness status, x= 12 - 16
#if (LCD_LINE_LENGTH != 16)
				lcd_putchar(' ');		// space
				lcd_putchar(1);			// speaker sign
#endif

#endif  //(LCD_LINES == 4)

//--------------------------------------------------------------------------------------------------------------------
				lcd_putchar(0);			// Volume indicator on last position
#endif  //ENABLE_LCD
				nDispUpd = 0;
/*
#if ! ((F_VS1001 == (6144 + 0x8000)) || (F_VS1001 == 12288) || defined(OLD_VS1001))
				if (vs1001_read(VS1001_CLOCKF) != F_VS1001)	// check for self reset of vs1001
					vs1001_reset(bLoudness);		// soft resetting for right CLOCKF set
#endif
*/
			} // if update time display
		} // if (bPlaying)
	} // while (true)

#endif // if not SETUP_REMOTE_CODES

}




// Auxillary functions definitions

// converts 16 bit unsigned integer to string
// str[] should be 6 bytes at least. Function can return value != str
// and you have to use it as a output value start
u08* unsigned2str(u16 track, u08 *str)
{
	register u08 i=5;
	udiv_t divt;

	while(i--) *str++ = ' ';
	*str = '\0';
	divt.quot = track;
	do {
		divt = udiv(divt.quot, 10u);
		*(--str) = '0' + (u08)divt.rem;
	} while (divt.quot);
	return str;
}

#ifdef ENABLE_NUMERIC
u16 Numb2Int()				// Convert selected song number from string to integer
{
register u08 i;
u16 res = 0;
u16 p   = 1;
	for (i = pNum-number ; i>0 ;  i--)
	{
		res += p * (number[i-1]-'0');
		p *=10;
	}
	return res;
}
#endif

#ifdef OLD_VS1001
void BadHeadStrip(void)		// Special MP3 header patch to strip all invalid headers 
				// not needed for VS1001K chip
{
u16 j;
	for (j = 0; j < BUFFER_SIZE - 1; j++)
		if ((*(u16*)&BUFFER1P[j] & 0xfeff) != 0xfaff)
			BUFFER1P[j] = 0;
		else
			break;
	// End of special header patch
}
#endif


#ifdef ENABLE_SERIAL
typedef struct pairUART				// auxilliary structure for making
{						//  of pairs table for select operator
	u08 code, action;
} pairUART;

pairUART TableUART[] = { 
	{ 0, EV_IDLE }, 
	{ 'g', EV_PLAY },	{ 'G', EV_STOP }, 
	{ 'p', EV_PREV },	{ 'n', EV_NEXT }, 
	{ 'P', EV_PREV10 },  	{ 'N', EV_NEXT10 },
	{ 'V', EV_VOLUP },	{ 'v', EV_VOLDN },
	{ 'd', EV_NEXTD },	{ 'D', EV_PREVD },
#ifdef ENABLE_NAVL
	{ 'l', EV_NEXTL },	{ 'L', EV_PREVL },
#endif
#ifdef FFWD_SEARCH
	{ 'f', EV_FFWD },
#endif
#ifdef FREW_SEARCH
	{ 'F', EV_FREW },
#endif
	{ 'r', EV_RANDOM },	{ 'b', EV_LOUDNESS},
	{ 't', EV_REMAIN} 
};


u08 get_char_event(void)
{
#ifdef HEXDUMP
	static u32 dwLbaSector = 0;
	static u32 dwTempAcc = 0;
#endif

	register u08 i;
	register u08 code = UART_ReceiveByte();

	for (i = 0; i < sizeof(TableUART) / sizeof(TableUART[0]); i++)
		if (TableUART[i].code == code)
			return TableUART[i].action;
#ifdef HEXDUMP
	if (code >= '0' && code <= '9')
		dwTempAcc = dwTempAcc * 10 + code - '0';
	if (code == 0x0d)		// Enter
	{
		sectordump(dwLbaSector = dwTempAcc);
		dwTempAcc = 0;
	}
	if (code == '+')
		sectordump(++dwLbaSector);
	if (code == '-')
		sectordump(--dwLbaSector);
#endif	// HEXDUMP

    return EV_IDLE;
}

#endif // ENABLE_SERIAL


#ifdef ENABLE_IR

// get a remote control event

u16 TableIR[] = {IR_PLAY, IR_STOP, IR_NEXT, IR_PREV, IR_NEXT10, IR_PREV10, IR_VOLUP, IR_VOLDN, IR_NEXTD, 
		IR_PREVD, IR_FFWD, IR_FREW, IR_LOUDNESS, IR_RANDOM, IR_REMAIN, IR_NEXTL, IR_PREVL};

#ifdef ENABLE_NUMERIC
 u16 NumTab[] = {IR_0, IR_1, IR_2, IR_3, IR_4, IR_5, IR_6, IR_7, IR_8, IR_9};
#endif

u08 ir_event(void)
{
	static u16 Lcod;
	u08 i;
	u16 code = get_rec80();

	if (Lcod == code && nKeyTime <= DOUBLE_TRAP)		// Trap for some IR transmitters that fast repeat
		return EV_IDLE;					// code of pressed key

	for (i = 0; i < (sizeof TableIR / sizeof TableIR[0]); i++)
		if (TableIR[i] == code)
		{
			LastEvent = i+1;				// Save last event
RET_LAST:		nKeyTime = 0;
			Lcod = code;
			return LastEvent;
		}

	if (code == IR_RESET)
	{
		wdt_enable(0x01);
		while(true);					// Reset player by watchdog
	}

#ifdef ENABLE_AUTOREPEAT
	if ((nKeyTime >= 2) && (LastEvent == EV_VOLUP || LastEvent == EV_VOLDN || 
				LastEvent == EV_FFWD  || LastEvent == EV_FREW))		// Autorepeat feature on some keys
		goto RET_LAST;
#endif

#ifdef ENABLE_NUMERIC
	for (i=0; i<10; i++)
		if (NumTab[i] == code)			// Check for numeric keypress code
		{
			*pNum++ = i+'0';		// Add numeric key ascii character to string
			*pNum = 0;			// String end flag
			LastEvent = EV_NUMBER;
			goto RET_LAST;
		}
			
#endif
	return EV_IDLE;
}

#else
 #define ir_event() EV_IDLE
#endif // ENABLE_IR


// get a keyboard event - edhanced by MIS

#ifdef ENABLE_LOCAL_KBD
u08 TableKbdShort[] = KBD_SHORT;	// Key definitions moved to "Constants.h" file !!!
u08 TableKbdLong[] = KBD_LONG;
#endif

u08 get_key_event(void)
{
	u08 event = EV_IDLE;
#ifdef ENABLE_LOCAL_KBD
	static u08 keydown = 0;
	register u08 i;
	disable_ram();			// disable ExtRAM
	outp(0xff, PORTA-1);		// set port as output
	outp(0, PORTA);			// set all pins to 0
	asm volatile("nop");		// allow time for port to activate

	if (keydown)			// if a key was pressed earlier
	{
		// see if it has been released
		if (bit_is_set(PIND, PD3))				// check bit
		{
			// no key is down, return last key seen
			if (nKeyTime < 8 && nKeyTime > 0)		// max 0.8 sec for shortpress
				event = TableKbdShort[keydown];		// short keypress event
			keydown = 0;
		}
		else
			if ((nKeyTime & 0x7F) >= 8)			// long keypress ? 
			{
				event = TableKbdLong[keydown];		// long keypress event
				if (event == EV_VOLUP || event == EV_VOLDN || 
				    event == EV_FFWD  || event == EV_FREW)
					nKeyTime = 0x86;		// 0.2 sec repeat time for volume and fast forward
				else
					nKeyTime = 0x82;		// 0.6 sec repeat time
			}	


	}
	else
	{	
		if (bit_is_clear(PIND, PD3))	// check if a key is down
		{
			// a key is active, check which one
			for (i = 0; i < 8; i++)
			{
				outp(~(1 << i), PORTA);			// write bit mask
				asm volatile("nop");			// wait a while
				asm volatile("nop");
				if (bit_is_clear(PIND, PD3)) 		// this bit ?
				{
					keydown = i + 1;		// store the key
					nKeyTime = 0;			// clear keypress timer
					break;				// and exit
				}
			}
		}
	}

	enable_ram();							// enable ExtRAM
#endif
	return event;
}

#ifdef PWR_BEEPS
void send_sinewave_beeps(void)
{
	static u08 buf[] = {0x53,0xEF,0x6E,0x30,0x45,0x78,0x69,0x74};
	register u08 i, j = 3;

	while (j-- != 0)
	{
		for (i = 0; i < 4; i++)
			vs1001_send_data(buf[i]);
		vs1001_nulls(4);
		delay10(10000);		// delay 100ms

		for (/*i = 4*/; i < 8; i++)
			vs1001_send_data(buf[i]);
		vs1001_nulls(4);
		delay10(10000);
	}	
}
#endif

#ifdef ENABLE_LCD
void defchar(u08 dat)
{
	register u08 i=5;
	lcd_data(0x1f);
	while (i--)
		lcd_data(dat);
	lcd_data(0x1f);
	lcd_data(0x00);				// cursor line
}
#endif

void setvolume(u08 v)
{
	vs1001_setvolume(v, v);
	eeprom_wb(EEPROM_VOLUME, v);

#ifdef ENABLE_LCD

	register u08 i;
#ifndef PROGRESBAR_II
	static u08 data[] = { 0x10, 0x18, 0x1C, 0x1E, 0x1F };
#endif
	lcd_command(0x40);
	for (i = 0; i < 8; i++)
	{
		if (v <= i * (VOL_STEP*(NUM_VOL_STEPS/8)))
#if (LCD_LINE_LENGTH == 16)
			lcd_data((1 << ((10 - i) / 2)) - 1);		// Volume idicator mod
#else
			lcd_data(0x1f << ((i+1)*5 / 10));		// Volume idicator mod (reversed)
#endif
		else
			lcd_data(0);
	}

#if (LCD_LINE_LENGTH != 16)
	static u08 vsc[] = {0x03, 0x05, 0x19, 0x19, 0x19, 0x05, 0x03, 0x00};	// speaker sign
	for (i=0; i < 8; i++)
		lcd_data(vsc[i]);					// load to CG_RAM char 1
#endif

#ifdef PROGRESBAR_II
	lcd_command(0x80);
#else
	lcd_command(0x50);
	for (i = 0; i < 40; i++)
		if ((i % 8) < 7) 
			lcd_data(data[i >> 3]);			// eight times send each value
		else
			lcd_data(0);
#endif

#endif // ENABLE_LCD
}

#ifdef PROGRESBAR_II
void lcd_frame(void)
{
	register u08 i = (LCD_STEPS / 5) - 1;
	lcd_command(0x58);			// start definition of CG-RAM from char code 3
	defchar(0x1f);				// full  bar (3)
	defchar(0x00);				// empty bar (4)
	defchar(0x01);				// ended bar (5)
	lcd_gotoxy(0, LCD_LINES / 4 + 1);
	while (i--)
		lcd_putchar(4);	
	lcd_putchar(5);	
}
#endif


u08 barcalc(u08 dummy)				// assembler optimized function to calculate progressbar position
{
// (u08)(LCD_STEPS * (dwPlayed>>8) / (dwFileSize>>8));

	asm volatile ("mov r18, r24");
	asm volatile ("clr r19");
	asm volatile ("clr r20");
	asm volatile ("clr r21");
	asm volatile ("lds r22,(dwPlayed)+1");
	asm volatile ("lds r23,(dwPlayed)+2");
	asm volatile ("lds r24,(dwPlayed)+3");
	asm volatile ("clr r25");
#ifdef YAMPP3USB
	asm volatile ("call __mulsi3");
#else
	asm volatile ("rcall __mulsi3");
#endif
	asm volatile ("lds r18,(dwFileSize)+1");
	asm volatile ("lds r19,(dwFileSize)+2");
	asm volatile ("lds r20,(dwFileSize)+3");
	asm volatile ("clr r21");
#ifdef YAMPP3USB
	asm volatile ("call __udivmodsi4");
#else
	asm volatile ("rcall __udivmodsi4");
#endif
	asm volatile ("mov r24,r18");
	return dummy;
}

void dispbar(void)
{
#ifdef ENABLE_LCD

	register u08 i, a, nSteps;

#ifdef PROGRESBAR_II
	nSteps = barcalc(LCD_STEPS-1);
	a = i = nSteps / 5;					// # of full bars
	static u08 data[] = { 0x10, 0x18, 0x1C, 0x1E, 0x1F };

	lcd_command(0x50);
	defchar(data[nSteps % 5] | (a == (LCD_STEPS / 5) - 1));

	lcd_gotoxy(0,LCD_LINES / 4 + 1);

	while (i--)
		lcd_putchar(3);					// print full bars

	if (a <= (LCD_STEPS / 5) - 1)				// jman to fix display overwrite
		lcd_putchar(2);	

#else // PROGRESBAR_II

	nSteps = barcalc(LCD_STEPS);
	a = i = nSteps / 5;					// # of full bars
	lcd_gotoxy(0, LCD_LINES / 4 + 1);			// line 2 or 3
	while(i--)
		lcd_putchar(6);

	if (a <= (LCD_STEPS / 5) - 1)				// jman to fix display overwrite
	{
		lcd_putchar(2 + nSteps % 5);	

#ifdef FREW_SEARCH
		if ( a < LCD_STEPS / 5)
			 lcd_putchar(32);			// space
#endif
	}
#endif // PROGRESBAR_II
#endif // ifdef ENABLE_LCD
}


void eeprom_ww(u08 address, u16 value)
{
	union u16convert Data;
	Data.value = value;
	eeprom_wb(address, Data.bytes.low);
	eeprom_wb(address + 1, Data.bytes.high);
}



#ifdef HEXDUMP

void HexDump(u08 *buffer, u16 len)
{
    u16 i, j, k;

    EOL();
    for (i = 0, k = 0; i < len / 16; i++, k += 16)
    {
        UART_Printfu16(k);
        UART_SendByte(' ');
        for (j = 0; j < 16; j++)
        {
            UART_Printfu08(buffer[i * 16 + j]);	
            UART_SendByte(' ');
        }
        PRINT("    ");
        for(j =0 ; j < 16; j++)
        {
            if (buffer[i * 16 + j] >= 0x20)
                UART_SendByte(buffer[i * 16 + j]);
            else
                UART_SendByte('.');
        }
        EOL();
    }
}

void sectordump(u32 sector)
{
	if (ATA_ReadLBA0(sector, 1, TMP_SECTOR) == 0)
	{
		UART_Printfu32(sector);
		HexDump(TMP_SECTOR, BPS);
	}
	else
		PRINT_p(PSTR("Dump failed\n"));
}

#endif

